﻿using System;
using System.Globalization;
using System.Threading.Tasks;
using Borepin.Service.Storage;
using Borepin.Service.Storage.Exceptions;

namespace Borepin.Model.Storage
{
    /// <summary>
    /// Store Credentials for Connection in SecureStorageService
    /// </summary>
    public class ConnectionCredentialStorage
    {
        #region Private Fields
        private readonly ISecretStorageService _SecretStorageService;
        #endregion

        #region Constructors
        public ConnectionCredentialStorage(ISecretStorageService secretService)
        {
            _SecretStorageService = secretService;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Get Password for Connection from SecureStorageService
        /// </summary>
        /// <exception cref="ArgumentException"></exception>
        /// <exception cref="MissingConnectionException"></exception>
        public async Task<string> GetPassword(Connection connection)
        {
            if (connection.AuthenticationTyp != AuthenticationTyp.PLAIN)
            {
                throw new ArgumentException("AuthenticationTyp is not PLAIN", nameof(connection));
            }

            string password = await _SecretStorageService.GetAsync(string.Format(CultureInfo.InvariantCulture, "bffh_password_{0}_{1}", connection.Address.ToString(), connection.Username)).ConfigureAwait(false);
            if (password == null)
            {
                throw new MissingConnectionException();
            }

            return password;
        }

        /// <summary>
        /// Add Password for Connection to SecureStorageService
        /// </summary>
        public async Task AddCredentials(Connection connection, string password)
        {
            await _SecretStorageService.SetAsync(string.Format(CultureInfo.InvariantCulture, "bffh_password_{0}_{1}", connection.Address.ToString(), connection.Username), password).ConfigureAwait(false);
        }

        /// <summary>
        /// Remove Password for Connection from SecureStorageService
        /// </summary>
        public Task RemoveCredentials(Connection connection)
        {
            _SecretStorageService.Remove(string.Format(CultureInfo.InvariantCulture, "bffh_password_{0}_{1}", connection.Address.ToString(), connection.Username));

            return Task.CompletedTask;
        }

        /// <summary>
        /// Remove all Connections from SecureStorage
        /// </summary>
        public Task RemoveAllCredentials()
        {
            _SecretStorageService.RemoveAll();

            return Task.CompletedTask;
        }
        #endregion
    }
}
