﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Borepin.Service.Storage;
using Borepin.Service.Storage.Exceptions;
using Newtonsoft.Json;

namespace Borepin.Model.Storage
{
    /// <summary>
    /// Store Connection in PreferenceStorageService
    /// </summary>
    public class ConnectionStorage
    {
        #region Private Fields
        private readonly IPreferenceStorageService _PreferenceService;
        #endregion

        #region Constructors
        public ConnectionStorage(IPreferenceStorageService preferenceService)
        {
            _PreferenceService = preferenceService;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Get Connection List from Storage
        /// </summary>
        /// <returns></returns>
        public Task<IList<Connection>> GetConnectionList()
        {
            return Task.FromResult(_LoadConnectionFormStorage());
        }

        /// <summary>
        /// Add Connection to Storage
        /// </summary>
        /// <exception cref="DuplicateConnectionException"></exception>
        public Task<bool> AddConnection(Connection connection)
        {
            IList<Connection> connection_list = _LoadConnectionFormStorage();

            if (connection_list.Contains(connection))
            {
                throw new DuplicateConnectionException();
            }

            connection_list.Add(connection);

            _SaveConnectionToStorage(connection_list);

            return Task.FromResult(true);
        }

        /// <summary>
        /// Remove Connection from Storage
        /// </summary>
        public Task<bool> RemoveConnection(Connection connection)
        {
            IList<Connection> connection_list = _LoadConnectionFormStorage();

            if (!connection_list.Contains(connection))
            {
                throw new MissingConnectionException();
            }

            while(connection_list.Contains(connection))
            {
                connection_list.Remove(connection);
            }
            
            _SaveConnectionToStorage(connection_list);

            return Task.FromResult(true);
        }

        /// <summary>
        /// Remove All Connection from Storage
        /// </summary>
        public Task<bool> RemoveAllConnections()
        {
            _SaveConnectionToStorage(new List<Connection>());

            return Task.FromResult(true);
        }

        /// <summary>
        /// Update Connections Timestamp in Storage 
        /// </summary>
        /// <exception cref="MissingConnectionException"></exception>
        public Task<bool> UpdateConnectionTimestamp(Connection connection)
        {
            IList<Connection> connection_list = _LoadConnectionFormStorage();

            if (!connection_list.Contains(connection))
            {
                throw new MissingConnectionException();
            }

            int index = connection_list.IndexOf(connection);
            Connection connection_update = connection_list[index];
            connection_update.LastTime = DateTime.UtcNow;
            connection_list[index] = connection_update;

            _SaveConnectionToStorage(connection_list);

            return Task.FromResult(true);
        }
        #endregion

        #region Private Methodss
        private IList<Connection> _LoadConnectionFormStorage()
        {
            List<Connection> connection_list;
            try
            {
                connection_list = JsonConvert.DeserializeObject<List<Connection>>(_PreferenceService.Get("connection_list", "[]"));
            }
            catch (JsonSerializationException)
            {
                connection_list = new List<Connection>();
                _PreferenceService.Set("connection_list", JsonConvert.SerializeObject(connection_list));
            }

            return connection_list;
        }

        private void _SaveConnectionToStorage(IList<Connection> connection_list)
        {
            _PreferenceService.Set("connection_list", JsonConvert.SerializeObject(connection_list));
        }
        #endregion
    }
}
