﻿using Borepin.Base;
using Borepin.Model;
using Borepin.Service.BFFH;
using Borepin.Service.BFFH.Exceptions;
using Prism.Commands;
using Prism.Navigation;
using Prism.Services;
using Prism.Services.Dialogs;
using System;
using System.Globalization;
using System.Threading.Tasks;
using System.Windows.Input;

namespace Borepin.PageModel
{
    public class ServerPageModel : PageModelBase
    {
        #region Private Fields
        private readonly IDialogService _DialogService;
        private readonly IBFFHService _BFFHService;
        private readonly IPageDialogService _PageDialogService;
        #endregion

        #region Constructors
        public ServerPageModel(INavigationService navigationService, IDialogService dialogService, IBFFHService bffhService, IPageDialogService pageDialogService) : base(navigationService)
        {
            _DialogService = dialogService;

            _BFFHService = bffhService;
            _PageDialogService = pageDialogService;

            ConnectCommand = new DelegateCommand(async () => await ConnectCommandExecute().ConfigureAwait(false));
            DisconnectCommand = new DelegateCommand(async () => await DisonnectCommandExecute().ConfigureAwait(false));
            DeleteCommand = new DelegateCommand(DeleteCommandExecute);
        }
        #endregion

        #region Data
        public override Task LoadData()
        {
            if (_BFFHService.CurrentConnection != null && Connection_Item != null)
            {
                InstanceIsActiveConnection = Connection_Item.Equals(_BFFHService.CurrentConnection);
            }
            else
            {
                InstanceIsActiveConnection = false;
            }

            if(_Connection_Item != null && _Connection_Item.Address != null)
            {
                DisplayAddress = string.Format(CultureInfo.InvariantCulture, "{0}:{1}", _Connection_Item.Address.Host, _Connection_Item.Address.Port);
            }

            IsBusy = false;
            return Task.CompletedTask;
        }
        #endregion

        #region Fields
        private Connection _Connection_Item;
        public Connection Connection_Item
        {
            get => _Connection_Item;
            set => SetProperty(ref _Connection_Item, value);
        }

        private string _DisplayAddress;
        public string DisplayAddress
        {
            get => _DisplayAddress;
            set => SetProperty(ref _DisplayAddress, value);
        }

        private bool _InstanceIsActiveConnection;
        public bool InstanceIsActiveConnection
        {
            get => _InstanceIsActiveConnection;
            set => SetProperty(ref _InstanceIsActiveConnection, value);
        }
        #endregion

        #region Commands
        private ICommand _ConnectCommand;
        public ICommand ConnectCommand
        {
            get => _ConnectCommand;
            set => SetProperty(ref _ConnectCommand, value);
        }
        public async Task ConnectCommandExecute()
        {
            IsBusy = true;

            if(_BFFHService.IsConnected)
            {
                await _BFFHService.Disconnect().ConfigureAwait(true);
            }

            try
            {
                await _BFFHService.Connect(Connection_Item).ConfigureAwait(true);
            }
            catch(ConnectingFailedException)
            {
                await _PageDialogService.DisplayAlertAsync("Connection failed", "Unable to connect to server.", "Ok").ConfigureAwait(false);
                    
                IsBusy = false;
                return;
            }
            catch(AuthenticatingFailedException)
            {
                await _PageDialogService.DisplayAlertAsync("Connection failed", "Unable to authenticate to server.", "Ok").ConfigureAwait(false);

                IsBusy = false;
                return;
            }
            catch(MissingCredentialsException)
            {
                await _PageDialogService.DisplayAlertAsync("Secure Storage failed", "Unable to load connections from secure storage.\n Please recreate the connection.", "Ok").ConfigureAwait(false);

                await _NavigationService.NavigateAsync("/MainPage/NavigationPage/ServerListPage").ConfigureAwait(false);
                IsBusy = false;
                return;
            }

            await _NavigationService.NavigateAsync("/MainPage/NavigationPage/MachineListPage").ConfigureAwait(false);
        }

        private ICommand _DisconnectCommand;
        public ICommand DisconnectCommand
        {
            get => _DisconnectCommand;
            set => SetProperty(ref _DisconnectCommand, value);
        }
        public async Task DisonnectCommandExecute()
        {
            await _BFFHService.Disconnect().ConfigureAwait(false);

            await LoadData().ConfigureAwait(false);
        }

        private ICommand _DeleteCommand;
        public ICommand DeleteCommand
        {
            get => _DeleteCommand;
            set => SetProperty(ref _DeleteCommand, value);
        }

        public void DeleteCommandExecute()
        {
            DialogParameters parameters = new DialogParameters
            {
                { "title", "Delete Server" },
                { "message", "Do you really want to delete this Server?" },
                { "instance", Connection_Item },
            };

            _DialogService.ShowDialog("ConfirmDialog", parameters, DeleteCommandExecute_Dialog);
        }

        public async void DeleteCommandExecute_Dialog(IDialogResult result)
        {
            if(string.Equals(result.Parameters.GetValue<string>("result"), "confirm", StringComparison.Ordinal))
            {
                Connection connection = result.Parameters.GetValue<Connection>("instance");

                await _BFFHService.RemoveConnection(connection).ConfigureAwait(false);

                await _NavigationService.NavigateAsync("/MainPage/NavigationPage/ServerListPage").ConfigureAwait(false);
            }
        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {
            
        }

        public override async void OnNavigatedTo(INavigationParameters parameters)
        {
            if(Connection_Item == null)
            {
                Connection_Item = parameters["instance"] as Connection;

                await LoadData().ConfigureAwait(false);
            }
        }
        #endregion
    }
}
